//
//  HousingApi.m
//  HousingApi
//
//  Created by sanaeg on 2015/01/23.
//  Copyright (c) 2015年 DaiwaHouse. All rights reserved.
//

#import "HousingApi.h"
#import "LocalXmlPaeserDelegate.h"
#import "HousingApiConstant.h"
#import "LocalResponse.h"
#import "CommonUtil.h"

NSString* const urlFormat = @"http://%@:%@%@";

@implementation HousingApi {
    LocalResponse* response;
    NSString* statusCode;
}

// HTTPリクエストの送信
- (BOOL)sendCommand: (NSString*)ipAddress port:(NSString*)portNo command:(NSString*)command timeoutInterval:(NSTimeInterval)timeoutInterval error:(NSError**)error
{
    *error = nil;
    statusCode = nil;
    NSHTTPURLResponse* res = nil;
    response = nil;
    
    BOOL result = YES;
    @try
    {
        NSString* pstrUrlString = [NSString stringWithFormat:urlFormat, ipAddress, portNo, command];
        
        // リクエスト
        NSURLRequest* pUrlRequest = [NSURLRequest requestWithURL: [NSURL URLWithString:pstrUrlString]
                                                     cachePolicy: NSURLRequestReloadIgnoringLocalCacheData
                                                 timeoutInterval: timeoutInterval];
        
        // XMLデータを取得
        NSData* pXmlData = nil;
        if (nil != pUrlRequest)
        {
            pXmlData = [NSURLConnection sendSynchronousRequest: pUrlRequest returningResponse:&res error: error];
        }
        
        if (nil != pXmlData && [pXmlData length] > 0)
        {
            // NSXMLParserオブジェクト生成
            NSXMLParser* pXMLParser = [[NSXMLParser alloc]initWithData: pXmlData];
            // XML解析のデリゲート生成
            LocalXmlPaeserDelegate* delegate = [[LocalXmlPaeserDelegate alloc] init];
            [pXMLParser setDelegate: delegate];
            [pXMLParser parse];
            response = [[LocalResponse alloc] initWithResponse:delegate.response];
        } else {
            if (*error == nil) {
                NSInteger status = [res statusCode];
                NSString* message = [NSHTTPURLResponse localizedStringForStatusCode:status];
                NSDictionary* userInfo = @{NSLocalizedDescriptionKey : message};
                
                *error = [NSError errorWithDomain:@"jp.co.daiwaHouse.housingApi" code:status userInfo:userInfo];
            }
            result = NO;
        }
    }
    @catch (NSException * e)
    {
        NSDictionary* userInfo = @{NSLocalizedDescriptionKey : @"不明なエラーが発生しました。設定値を確認してください。",
                                   NSLocalizedRecoverySuggestionErrorKey : e};
        *error = [NSError errorWithDomain:@"jp.co.daiwaHouse.housingApi" code:-1 userInfo:userInfo];
        result = NO;
    }
    
    return result;
}

- (BOOL) getResponseResult:(NSString*)rootElement, ...NS_REQUIRES_NIL_TERMINATION {

    //可変長オブジェクトを取得
    va_list args;
    va_start(args, rootElement);
    va_end(args);

    if (response != nil) {
        return [response getResponseResult:rootElement arguments:args];
    } else {
        return false;
    }
}

- (id) getResponseData:(NSString*)rootElement, ...NS_REQUIRES_NIL_TERMINATION {

    //可変長オブジェクトを取得
    va_list args;
    va_start(args, rootElement);
    va_end(args);

    if (response != nil) {
        return [response getResponseData:rootElement arguments:args];
    } else {
        return nil;
    }
}

- (NSString*) getResponseError:(NSString*)rootElement, ...NS_REQUIRES_NIL_TERMINATION {

    //可変長オブジェクトを取得
    va_list args;
    va_start(args, rootElement);
    va_end(args);
    
    if (statusCode != nil) {
        return statusCode;
    } else {
        if (response != nil) {
            return [response getResponseData:rootElement arguments:args];
        } else {
            return @"-9999";
        }
    }
}

@end
